﻿local addonName, addonTable = ...

local C = addonTable.C
local L = addonTable.L

RegisterAddonMessagePrefix(C["prefix"])

local players_scanned = {}
local stage = 0
local ping_response = 6
local ping_timeout  = 16
addonTable.opponent = { connected = false,
                        name = "",
                        portrait = "",
                        score = 0 }
                        
local function SetOpponent(name)
 addonTable.opponent.connected = true
 addonTable.opponent.name = name
 addonTable.opponent.score = 0
end

local function SendMsg(s, name) -- string, name
 SendAddonMessage(C["prefix"], s, "WHISPER", name)
end

addonTable.SendToOpponent = function(t, arg)
if addonTable.opponent.connected and addonTable.opponent.name then
  SendMsg(t..arg, addonTable.opponent.name)
 end
end

addonTable.ResetMultiplayerInterface = function()
 stage = 0
 addonTable.Timer_Reset(4)
 addonTable.Timer_Reset(5)
 addonTable.opponent = { connected = false,
                         name = "", 
                         portrait = "",
                         score = 0 }
end

local function Ping(name)
 SendMsg(C["ping"], name)
 addonTable.Timer_Start(4,ping_timeout, function() addonTable.EndGame("pingtimeout") end, "Sound\\Doodad\\Goblin_Lottery_Open01.wav") -- ping timer
end

local function ProcessAddonMsg(pref, s, chan, sender)
 if sender == UnitName("player") then
  return
 end

 if pref ~= C["prefix"] then return end

 if strfind(s, C["verscheck"]) then
  if stage > 0 then return end --- double and useless version check.. temporary
  local v = tonumber(strmatch(s,"(%d+)"))
  if addonTable.version > v then
   SendMsg(C["vershigh"]..addonTable.version, sender)
   print(L["addontag"]..sender..L["dbhigh"])
  elseif addonTable.version < v then
   SendMsg(C["verslow"]..addonTable.version, sender)
   print(L["addontag"]..L["dblow"]..sender)
  else
   -- RECEIVER
   -- stage 1 : version check passed, now we check the database
   stage = 1
   SetOpponent(sender)
   SendMsg(C["dbcheck"]..table.getn(addonTable.wowzzle_database), sender)
  end
 elseif strfind(s, C["verslow"]) then 
  print(L["addontag"]..sender..L["dbhigh"])
 elseif strfind(s, C["vershigh"]) then
  print(L["addontag"]..L["dblow"]..sender)
 elseif strfind(s, C["decline"]) then
  print(L["addontag"]..sender..L["declined"])
 elseif strfind(s, C["gamereq"]) then
  if stage > 0 then return end
  -- SENDER
  local v = tonumber(strmatch(s,"(%d+)"))
  if addonTable.version > v then
   SendMsg(C["vershigh"]..addonTable.version, sender)
   print(L["addontag"]..sender..L["dbhigh"])
  elseif addonTable.version < v then
   SendMsg(C["verslow"]..addonTable.version, sender)
   print(L["addontag"]..L["dblow"]..sender)
  else
   addonTable.OpenGameRequestInterface(sender)
  end
 elseif strfind(s, C["dbcheck"]) then
  if stage > 1 then return end ---
  if table.getn(addonTable.wowzzle_database) ~= tonumber(strmatch(s,"(%d+)")) then
   SendMsg(C["dbdiff"], sender)
   print(L["addontag"]..L["dbdiff"])
  else
   -- SENDER
   -- stage 2 : database check passed, send a new grid
   stage = 2
   SetOpponent(sender)
   Ping(sender)
   addonTable.InitGame("multiplayer")
   SendMsg(C["grid"]..addonTable.GetGrid(), sender)
  end
 elseif strfind(s, C["grid"]) then
  if stage > 2 then return end
  local s1 = string.gsub(s, C["grid"], "")
  addonTable.InitGame("multiplayer")
  addonTable.SetGrid(s1)
  SendMsg(C["gridok"], sender)
  -- RECEIVER
  -- stage 3 : grid received
  Ping(sender)
  stage = 3
 elseif strfind(s, C["gridok"]) then
  -- SENDER
  -- stage 4 : start game
  stage = 4
  SendMsg(C["start"], sender)
  addonTable.StartGame()
 elseif strfind(s, C["scanreq"]) then
  local v = tonumber(strmatch(s,"(%d+)"))
  if v == addonTable.version then
   SendMsg(C["scanok"], sender)
  end
 elseif strfind(s, C["scanok"]) then
  players_scanned = sender.."\n"..players_scanned
  addonTable.Timer_Start(4,2, function()
    if addonTable.tmp_scanfunc then
     addonTable.tmp_scanfunc(players_scanned)
    end
    players_scanned = ""
   end)
 elseif strfind(s, C["ping"]) then
  if stage > 0 and addonTable.opponent.name == sender then
   addonTable.Timer_Start(4,ping_timeout, function() addonTable.EndGame("pingtimeout") end) -- ping timer
   addonTable.Timer_Start(5,ping_response, function() Ping(sender) end)
  end
 elseif strfind(s, C["start"]) then
  -- RECEIVER
  -- stage 4 : start game
  stage = 4
  addonTable.StartGame()
 elseif strfind(s, C["newscore"]) then
  if not Wowzzle_Mainframe then return end
  local s1 = tonumber(string.match(s,"(%d+)"))
  addonTable.opponent.score = addonTable.opponent.score + s1
  addonTable.CreateFloatingText(Wowzzle_Mainframe.rightframe, s1, 4)
  addonTable.RefreshSideFrames()
 elseif strfind(s, C["endresult"]) then
  local score = tonumber(string.match(s,"(%d+)@"))
  local words = string.match(s,"@(%D+)")
  if not words then words = {} end
  if not score then score = 0 end
  --words = string.gsub(words, ",", " ")
  addonTable.ShowGameResults("datareceived", score, words)
 end
end

addonTable.ScanForWowzzlePlayers = function(arg)
 players_scanned = ""
 SendAddonMessage(C["prefix"], C["scanreq"]..addonTable.version, "GUILD", "")
end

local eventframe = CreateFrame("Frame")
eventframe:RegisterEvent("CHAT_MSG_ADDON")
local function eventhandler(self, event, ...)
 if event == "CHAT_MSG_ADDON" then
  ProcessAddonMsg(...)
 end
end
eventframe:SetScript("OnEvent", eventhandler)