﻿--- Timer Functions ---

local addonName, addonTable = ...
local L = addonTable.L

local floor, abs = math.floor, math.abs

-- vars
local timers = {}
-- 1 - game words timer
-- 2 - comm timer
-- 3 - global timer
-- 4 - ping timeout timer
-- 5 - ping response timer

-- methods
addonTable.Timer_Running = function(t)
 return timers[t].running
end

addonTable.Timer_Reset = function(t)
 local timer = timers[t]
 if not timer then return end
 timer.Reset()
end

addonTable.ResetTimers = function()
 table.foreach(timers, function(k,v) v.Reset() end)
end

addonTable.Timer_Start = function(t,duration, func, sound)
 local timer = timers[t]
 if not timer then return end
 
 timer.pT_start = GetTime()
 timer.running = true
 timer:SetScript("OnUpdate",
   function()
    if t == 3 and Wowzzle_Mainframe then -- global timer
     local secs = abs(floor(GetTime() - timer.pT_start - duration))
     if prev ~= secs then
      local mins = 0
      while secs >= 60 do
       secs = secs - 60
       mins = mins + 1
      end
      if secs < 10 then
       if mins == 0 then
        secs = "|cffff00000"..secs.."|r"
       else
        secs = "0"..secs
       end
      end
      if secs == 10 and mins == 0 then
       PlaySoundFile("Sound\\Interface\\PVPWarningHordeMono.wav")
      end
      local formattedtime = mins > 0 and tostring(mins)..":"..secs or secs
      Wowzzle_Mainframe.globaltime:SetText(L["timeleft"]..formattedtime)
      prev = secs
     end
    end
    if GetTime() - timer.pT_start > duration then
     timer.Reset()
     if sound then
      PlaySoundFile(sound)
     end
     func()
    end
    end)
end

addonTable.Timer_Init = function()
 local i
 for i = 1,5 do
  local t = CreateFrame("FRAME")
  t.Reset = function()
   t.pT_start = 0
   t.running = false
   t:SetScript("OnUpdate", nil)
  end
  t.Reset()
  table.insert(timers, t)
 end
end